searchData={"items":[{"type":"module","doc":"# Xav\n\n[![Hex.pm](https://img.shields.io/hexpm/v/xav.svg)](https://hex.pm/packages/xav)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/xav)\n[![CI](https://img.shields.io/github/actions/workflow/status/elixir-webrtc/xav/ci.yml?logo=github&label=CI)](https://github.com/elixir-webrtc/xav/actions/workflows/ci.yml)\n[![codecov](https://codecov.io/gh/elixir-webrtc/xav/graph/badge.svg?token=2AG2acRhOf)](https://codecov.io/gh/elixir-webrtc/xav)\n\nElixir wrapper over FFmpeg for reading audio and video files.\n\nSee an interview with FFmpeg enthusiast:  https://youtu.be/9kaIXkImCAM","title":"Xav","ref":"Xav.html"},{"type":"module","doc":"Make sure you have installed FFMpeg (ver. 4.x - 7.x) development packages on your system\n(see [here](INSTALL.md) for installation one-liners) and add Xav to the list of your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:xav, \"~> 0.11.0\"},\n    # Add Nx if you want to have Xav.Frame.to_nx/1\n    {:nx, \">= 0.0.0\"}\n  ]\nend\n```","title":"Installation - Xav","ref":"Xav.html#module-installation"},{"type":"module","doc":"Decode\n\n```elixir\ndecoder = Xav.Decoder.new(:vp8, out_format: :rgb24)\n{:ok, %Xav.Frame{} = frame} = Xav.Decoder.decode(decoder, <<\"somebinary\">>)\n```\n\nDecode with audio resampling\n\n```elixir\ndecoder = Xav.Decoder.new(:opus, out_format: :flt, out_sample_rate: 16_000)\n{:ok, %Xav.Frame{} = frame} = Xav.Decoder.decode(decoder, <<\"somebinary\">>)\n```\n\nRead from a file:\n\n```elixir\nr = Xav.Reader.new!(\"./some_mp4_file.mp4\")\n{:ok, %Xav.Frame{} = frame} = Xav.Reader.next_frame(r)\ntensor = Xav.Frame.to_nx(frame)\nKino.Image.new(tensor)\n```\n\nRead from a camera:\n\n```elixir\nr = Xav.Reader.new!(\"/dev/video0\", device?: true, out_format: :rgb24)\n{:ok, %Xav.Frame{} = frame} = Xav.Reader.next_frame(r)\ntensor = Xav.Frame.to_nx(frame)\nKino.Image.new(tensor)\n```\n\nSpeech to text:\n\n```elixir\n{:ok, whisper} = Bumblebee.load_model({:hf, \"openai/whisper-tiny\"})\n{:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"openai/whisper-tiny\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/whisper-tiny\"})\n{:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai/whisper-tiny\"})\n\nserving =\n  Bumblebee.Audio.speech_to_text_whisper(whisper, featurizer, tokenizer, generation_config,\n    defn_options: [compiler: EXLA]\n  )\n\n# Read a couple of frames.\n# See https://hexdocs.pm/bumblebee/Bumblebee.Audio.WhisperFeaturizer.html for default sampling rate.\nframes =\n    Xav.Reader.stream!(\"sample.mp3\", read: :audio, out_format: :flt, out_channels: 1, out_sample_rate: 16_000)\n    |> Stream.take(200)\n    |> Enum.map(fn frame -> Xav.Frame.to_nx(frame) end)\n\nbatch = Nx.Batch.concatenate(frames)\nbatch = Nx.Defn.jit_apply(&Function.identity/1, [batch])\nNx.Serving.run(serving, batch) \n```","title":"Usage - Xav","ref":"Xav.html#module-usage"},{"type":"module","doc":"To make `clangd` aware of the header files used in your project, you can create a `compile_commands.json` file. \n`clangd` uses this file to know the compiler flags, include paths, and other compilation options for each source file.","title":"Development - Xav","ref":"Xav.html#module-development"},{"type":"module","doc":"The easiest way to generate `compile_commands.json` from a Makefile is to use the `bear` tool. `bear` is a tool that records the compiler calls during a build and creates the `compile_commands.json` file.\n\nYou can install `bear` with your package manager:\n\n- __macOS__: brew install bear\n- __Ubuntu/Debian__: sudo apt install bear\n- __Fedora__: sudo dnf install bear","title":"Install bear - Xav","ref":"Xav.html#module-install-bear"},{"type":"module","doc":"After installing bear, you can run it alongside your make command to capture the necessary information.\n\n```bash\nbear -- mix compile\n```","title":"Generate compile_commands.json - Xav","ref":"Xav.html#module-generate-compile_commands-json"},{"type":"function","doc":"List all decoders.","title":"Xav.list_decoders/0","ref":"Xav.html#list_decoders/0"},{"type":"function","doc":"List all encoders.","title":"Xav.list_encoders/0","ref":"Xav.html#list_encoders/0"},{"type":"function","doc":"Get all available pixel formats.\n\nThe result is a list of 3-element tuples `{name, nb_components, hw_accelerated_format?}`:\n  * `name` - The name of the pixel format.\n  * `nb_components` - The number of the components in the pixel format.\n  * `hw_accelerated_format?` - Whether the pixel format is a hardware accelerated format.","title":"Xav.pixel_formats/0","ref":"Xav.html#pixel_formats/0"},{"type":"function","doc":"Get all available audio sample formats.\n\nThe result is a list of 2-element tuples `{name, nb_bytes}`:\n  * `name` - The name of the sample format.\n  * `nb_bytes` - The number of bytes per sample.","title":"Xav.sample_formats/0","ref":"Xav.html#sample_formats/0"},{"type":"type","doc":"","title":"Xav.decoder/0","ref":"Xav.html#t:decoder/0"},{"type":"type","doc":"","title":"Xav.encoder/0","ref":"Xav.html#t:encoder/0"},{"type":"module","doc":"Audio/video decoder.","title":"Xav.Decoder","ref":"Xav.Decoder.html"},{"type":"function","doc":"Decodes an audio/video frame.\n\nSome video frames are meant for decoder only and will not\ncontain actual video samples.\nSome audio frames might require more data to be converted\nto the desired output format.\nIn both cases, `:ok` term is returned and more data needs to be provided.","title":"Xav.Decoder.decode/3","ref":"Xav.Decoder.html#decode/3"},{"type":"function","doc":"Flushes the decoder.\n\nFlushing signals end of stream and forces the decoder to return\nthe buffered frames if there're any.","title":"Xav.Decoder.flush/1","ref":"Xav.Decoder.html#flush/1"},{"type":"function","doc":"Same as `flush/1` but raises an exception on error.","title":"Xav.Decoder.flush!/1","ref":"Xav.Decoder.html#flush!/1"},{"type":"function","doc":"Creates a new decoder.\n\n`codec` is any audio/video decoder supported by `FFmpeg`.\n\n`opts` can be used to specify desired output parameters:\n* `:channels` (`t:pos_integer/0`) - The number of channels of the encoded audio.\n\n  Some decoders require this field to be set by the user. (e.g. `G711`)\n\n* `:out_format` (`t:atom/0`) - Output format of the samples.\n\n  In case of video, it's the pixel format. In case of audio, it's the sample format.\n\n  To get the list of supported pixel formats use `Xav.pixel_formats/0`,\n  and for sample formats `Xav.sample_formats/0`.\n\n* `:out_sample_rate` (`t:pos_integer/0`) - Audio sample rate.\n\n  If not specified, the sample rate of the input stream will be used.\n\n* `:out_channels` (`t:pos_integer/0`) - Number of audio channels.\n\n  If not specified, the number of channels of the input stream will be used.\n\n  Audio samples are always in the packed form -\n  samples from different channels are interleaved in the same, single binary:\n\n  ```\n  < >\n  ```\n\n  An alternative would be to return a list of binaries, where\n  each binary represents different channel:\n  ```\n  [\n    < >,\n    < >,\n    < >\n  ]\n  ```\n\n* `:out_width` (`t:pos_integer/0`) - Scale the output video frame to the provided width.\n\n* `:out_height` (`t:pos_integer/0`) - Scale the output video frame to the provided height.","title":"Xav.Decoder.new/2","ref":"Xav.Decoder.html#new/2"},{"type":"type","doc":"Supported codecs.\n\nTo get the list of available decoders see `Xav.list_decoders/0`.","title":"Xav.Decoder.codec/0","ref":"Xav.Decoder.html#t:codec/0"},{"type":"type","doc":"","title":"Xav.Decoder.t/0","ref":"Xav.Decoder.html#t:t/0"},{"type":"module","doc":"Audio/Video encoder.\n\nCurrently, it only supports video encoding.","title":"Xav.Encoder","ref":"Xav.Encoder.html"},{"type":"function","doc":"Encodes a frame.\n\nThe return value may be an empty list in case the encoder\nneeds more frames to produce a packet.","title":"Xav.Encoder.encode/2","ref":"Xav.Encoder.html#encode/2"},{"type":"function","doc":"Flush the encoder.","title":"Xav.Encoder.flush/1","ref":"Xav.Encoder.html#flush/1"},{"type":"function","doc":"Create a new encoder.\n\nTo get the list of available encoders, see `Xav.list_encoders/0`.\n\nIt accepts the following options:\n* `:width` (`t:pos_integer/0`) - Required. Width of the video samples.\n\n* `:height` (`t:pos_integer/0`) - Required. Height of the video samples.\n\n* `:format` (`t:atom/0`) - Required. Pixel format of the video samples.\n\n* `:time_base` (tuple of `t:pos_integer/0`, `t:pos_integer/0` values) - Required. Time base of the video stream.\n\n  It is a rational represented as a tuple of two postive integers `{numerator, denominator}`.\n  It represent the number of ticks `denominator` in `numerator` seconds. e.g. `{1, 90000}` reprensents\n  90000 ticks in 1 second.\n\n  it is used for the decoding and presentation timestamps of the video frames. For video frames with constant\n  frame rate, choose a timebase of `{1, frame_rate}`.\n\n* `:gop_size` (`t:pos_integer/0`) - Group of pictures length.\n\n  Determines the interval in which I-Frames (or keyframes) are inserted in\n  the stream. e.g. a value of 50, means the I-Frame will be inserted at the 1st frame,\n  the 51st frame, the 101st frame, and so on.\n\n* `:max_b_frames` (`t:non_neg_integer/0`) - Maximum number of consecutive B-Frames to insert between non-B-Frames.\n\n  A value of 0, disable insertion of B-Frames.\n\n* `:profile` (`t:String.t/0`) - The encoder's profile.\n\n  A profile defines the capabilities and features an encoder can use to\n  target specific applications (e.g. `live video`)\n\n  To get the list of available profiles for an encoder, see `Xav.list_encoders/0`","title":"Xav.Encoder.new/2","ref":"Xav.Encoder.html#new/2"},{"type":"type","doc":"","title":"Xav.Encoder.codec/0","ref":"Xav.Encoder.html#t:codec/0"},{"type":"type","doc":"","title":"Xav.Encoder.encoder_options/0","ref":"Xav.Encoder.html#t:encoder_options/0"},{"type":"type","doc":"","title":"Xav.Encoder.t/0","ref":"Xav.Encoder.html#t:t/0"},{"type":"module","doc":"Audio/video frame.","title":"Xav.Frame","ref":"Xav.Frame.html"},{"type":"function","doc":"","title":"Xav.Frame.new/4","ref":"Xav.Frame.html#new/4"},{"type":"function","doc":"Creates a new audio/video frame.","title":"Xav.Frame.new/5","ref":"Xav.Frame.html#new/5"},{"type":"function","doc":"Converts a frame to an Nx tensor.\n\nIn case of a video frame, dimension names of the newly created tensor are `[:height, :width, :channels]`.\n\nFor video frames, the only supported pixel formats are:\n  * `:rgb24`\n  * `:bgr24`","title":"Xav.Frame.to_nx/1","ref":"Xav.Frame.html#to_nx/1"},{"type":"type","doc":"Possible audio samples formats.\n\nTo get the complete list of sample formats, check `Xav.sample_formats/0`.","title":"Xav.Frame.audio_format/0","ref":"Xav.Frame.html#t:audio_format/0"},{"type":"type","doc":"","title":"Xav.Frame.format/0","ref":"Xav.Frame.html#t:format/0"},{"type":"type","doc":"","title":"Xav.Frame.height/0","ref":"Xav.Frame.html#t:height/0"},{"type":"type","doc":"","title":"Xav.Frame.t/0","ref":"Xav.Frame.html#t:t/0"},{"type":"type","doc":"Possible video frame formats.\n\nTo get the complete list of pixel formats, check `Xav.pixel_formats/0`.\n\nAn example of a pixel format is `:rgb24`.","title":"Xav.Frame.video_format/0","ref":"Xav.Frame.html#t:video_format/0"},{"type":"type","doc":"","title":"Xav.Frame.width/0","ref":"Xav.Frame.html#t:width/0"},{"type":"module","doc":"A module representing an audio/video compressed data.","title":"Xav.Packet","ref":"Xav.Packet.html"},{"type":"function","doc":"","title":"Xav.Packet.new/1","ref":"Xav.Packet.html#new/1"},{"type":"type","doc":"","title":"Xav.Packet.t/0","ref":"Xav.Packet.html#t:t/0"},{"type":"module","doc":"Audio/video file reader.","title":"Xav.Reader","ref":"Xav.Reader.html"},{"type":"function","doc":"Creates a new audio/video reader.\n\nBoth reading from a file and from a video camera are supported.\nIn case of using a video camera, the v4l2 driver is required, and FPS are\nlocked to 10.\n\nMicrophone input is not supported.\n\nThe following options can be provided:\n* `:read` - The type of the stream to read from the input, either `video` or `audio` The default value is `:video`.\n\n* `:device?` (`t:boolean/0`) - Whether the path points to the camera The default value is `false`.\n\n* `:out_format` (`t:atom/0`) - The output format of the audio samples. For a list of available\n  sample formats check `Xav.sample_formats/0`.\n\n  For video samples, it is always `:rgb24`.\n\n* `:out_sample_rate` (`t:pos_integer/0`) - The output sample rate of the audio samples\n\n* `:out_channels` (`t:pos_integer/0`) - The output number of channels of the audio samples","title":"Xav.Reader.new/2","ref":"Xav.Reader.html#new/2"},{"type":"function","doc":"The same as `new/1` but raises on error.","title":"Xav.Reader.new!/2","ref":"Xav.Reader.html#new!/2"},{"type":"function","doc":"Reads and decodes the next frame.","title":"Xav.Reader.next_frame/1","ref":"Xav.Reader.html#next_frame/1"},{"type":"function","doc":"Seeks the reader to the given time in seconds","title":"Xav.Reader.seek/2","ref":"Xav.Reader.html#seek/2"},{"type":"function","doc":"Creates a new reader stream.\n\nCheck `new/1` for the available options.","title":"Xav.Reader.stream!/2","ref":"Xav.Reader.html#stream!/2"},{"type":"type","doc":"","title":"Xav.Reader.t/0","ref":"Xav.Reader.html#t:t/0"},{"type":"module","doc":"Video samples converter.\n\nIt supports pixel format conversion and/or scaling.","title":"Xav.VideoConverter","ref":"Xav.VideoConverter.html"},{"type":"function","doc":"Converts a video frame.","title":"Xav.VideoConverter.convert/2","ref":"Xav.VideoConverter.html#convert/2"},{"type":"function","doc":"Creates a new video converter.\n\nThe following options can be passed:\n* `:out_width` (`t:pos_integer/0`) - scale the video frame to this width\n\n  If `out_width` and `out_height` are both not provided, scaling is not performed. If one of the\n  dimensions is `nil`, the other will be calculated based on the input dimensions as\n  to keep the aspect ratio.\n\n* `:out_height` (`t:pos_integer/0`) - scale the video frame to this height\n\n* `:out_format` (`t:atom/0`) - video format to convert to (e.g. `:rgb24`)","title":"Xav.VideoConverter.new/1","ref":"Xav.VideoConverter.html#new/1"},{"type":"type","doc":"","title":"Xav.VideoConverter.t/0","ref":"Xav.VideoConverter.html#t:t/0"},{"type":"extras","doc":"# Xav\n\n[![Hex.pm](https://img.shields.io/hexpm/v/xav.svg)](https://hex.pm/packages/xav)\n[![API Docs](https://img.shields.io/badge/api-docs-yellow.svg?style=flat)](https://hexdocs.pm/xav)\n[![CI](https://img.shields.io/github/actions/workflow/status/elixir-webrtc/xav/ci.yml?logo=github&label=CI)](https://github.com/elixir-webrtc/xav/actions/workflows/ci.yml)\n[![codecov](https://codecov.io/gh/elixir-webrtc/xav/graph/badge.svg?token=2AG2acRhOf)](https://codecov.io/gh/elixir-webrtc/xav)\n\nElixir wrapper over FFmpeg for reading audio and video files.\n\nSee an interview with FFmpeg enthusiast:  https://youtu.be/9kaIXkImCAM","title":"Xav","ref":"readme.html"},{"type":"extras","doc":"Make sure you have installed FFMpeg (ver. 4.x - 7.x) development packages on your system\n(see [here](INSTALL.md) for installation one-liners) and add Xav to the list of your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:xav, \"~> 0.11.0\"},\n    # Add Nx if you want to have Xav.Frame.to_nx/1\n    {:nx, \">= 0.0.0\"}\n  ]\nend\n```","title":"Installation - Xav","ref":"readme.html#installation"},{"type":"extras","doc":"Decode\n\n```elixir\ndecoder = Xav.Decoder.new(:vp8, out_format: :rgb24)\n{:ok, %Xav.Frame{} = frame} = Xav.Decoder.decode(decoder, <<\"somebinary\">>)\n```\n\nDecode with audio resampling\n\n```elixir\ndecoder = Xav.Decoder.new(:opus, out_format: :flt, out_sample_rate: 16_000)\n{:ok, %Xav.Frame{} = frame} = Xav.Decoder.decode(decoder, <<\"somebinary\">>)\n```\n\nRead from a file:\n\n```elixir\nr = Xav.Reader.new!(\"./some_mp4_file.mp4\")\n{:ok, %Xav.Frame{} = frame} = Xav.Reader.next_frame(r)\ntensor = Xav.Frame.to_nx(frame)\nKino.Image.new(tensor)\n```\n\nRead from a camera:\n\n```elixir\nr = Xav.Reader.new!(\"/dev/video0\", device?: true, out_format: :rgb24)\n{:ok, %Xav.Frame{} = frame} = Xav.Reader.next_frame(r)\ntensor = Xav.Frame.to_nx(frame)\nKino.Image.new(tensor)\n```\n\nSpeech to text:\n\n```elixir\n{:ok, whisper} = Bumblebee.load_model({:hf, \"openai/whisper-tiny\"})\n{:ok, featurizer} = Bumblebee.load_featurizer({:hf, \"openai/whisper-tiny\"})\n{:ok, tokenizer} = Bumblebee.load_tokenizer({:hf, \"openai/whisper-tiny\"})\n{:ok, generation_config} = Bumblebee.load_generation_config({:hf, \"openai/whisper-tiny\"})\n\nserving =\n  Bumblebee.Audio.speech_to_text_whisper(whisper, featurizer, tokenizer, generation_config,\n    defn_options: [compiler: EXLA]\n  )\n\n# Read a couple of frames.\n# See https://hexdocs.pm/bumblebee/Bumblebee.Audio.WhisperFeaturizer.html for default sampling rate.\nframes =\n    Xav.Reader.stream!(\"sample.mp3\", read: :audio, out_format: :flt, out_channels: 1, out_sample_rate: 16_000)\n    |> Stream.take(200)\n    |> Enum.map(fn frame -> Xav.Frame.to_nx(frame) end)\n\nbatch = Nx.Batch.concatenate(frames)\nbatch = Nx.Defn.jit_apply(&Function.identity/1, [batch])\nNx.Serving.run(serving, batch) \n```","title":"Usage - Xav","ref":"readme.html#usage"},{"type":"extras","doc":"To make `clangd` aware of the header files used in your project, you can create a `compile_commands.json` file. \n`clangd` uses this file to know the compiler flags, include paths, and other compilation options for each source file.","title":"Development - Xav","ref":"readme.html#development"},{"type":"extras","doc":"The easiest way to generate `compile_commands.json` from a Makefile is to use the `bear` tool. `bear` is a tool that records the compiler calls during a build and creates the `compile_commands.json` file.\n\nYou can install `bear` with your package manager:\n\n- __macOS__: brew install bear\n- __Ubuntu/Debian__: sudo apt install bear\n- __Fedora__: sudo dnf install bear","title":"Install bear - Xav","ref":"readme.html#install-bear"},{"type":"extras","doc":"After installing bear, you can run it alongside your make command to capture the necessary information.\n\n```bash\nbear -- mix compile\n```","title":"Generate compile_commands.json - Xav","ref":"readme.html#generate-compile_commands-json"},{"type":"extras","doc":"Xav requires FFmpeg development packages to be installed on your system.\nYou can install them with one of the following one-liners.\n\nOn some platforms, we use pkg-config to determine ffmpeg's include and lib directories.\n\n**Fedora**\n\n```bash\ndnf install pkg-config ffmpeg-devel ffmpeg-libs\n```\n\n**Ubuntu**\n\n```bash\napt install libavcodec-dev libavformat-dev libavutil-dev libswscale-dev libavdevice-dev\n```\n\n**MacOS x86_64**\n\n```bash\nbrew install ffmpeg\n```\n\n**MacOS arm64**\n\n```bash\nbrew install pkg-config ffmpeg\n```\n\n**Windows**\n\nWindows is not supported but PRs are welcomed.","title":"INSTALL","ref":"install.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}